program postprocessing

  implicit none

  integer,parameter :: NPOINTS=371250
  double precision,dimension(NPOINTS) :: rho0,vp0,vs0,kappa0,mu0
  double precision,dimension(NPOINTS) :: rho,vp,vs,kappa,mu,weights
  double precision,dimension(NPOINTS) :: krhop,kalpha,kbeta,krho,kkappa,kmu
  double precision,dimension(NPOINTS) :: MODELSPACE_temp
  integer :: ios,irec,NSTEP,ipoint,iglob
  double precision :: DT,x,z,DATASPACE,MODELSPACE
  real(kind=4),dimension(:),allocatable :: adj
  integer(kind=4) :: r4head(60)
  integer(kind=4) :: header4(1)
  integer(kind=2) :: header2(2)
  equivalence(header2,header4)
  character(len=512) :: filename

  ! read NSTEP
  filename="Ux_file_single.bin"
  open(111,file="./SEM/"//trim(adjustl(filename))//".adj",access='direct',recl=240,iostat = ios)
           if (ios /= 0) stop 'file containing adjoint sources does not exist'
  read(111,rec=1,iostat=ios) r4head
           if (ios /= 0) stop 'read error adj'
  header4=r4head(29)
  NSTEP=header2(2)
  header4=r4head(30)
  DT=header2(1)/1.0d6
  print*, 'irec=',r4head(1)
  print*, 'xs=',r4head(19)
  print*, 'zs=',r4head(20)
  print*, 'xr=',r4head(21)
  print*, 'zr=',r4head(22)
  print*, 'NSTEP=',NSTEP
  print*, 'DT=', DT
  allocate(adj(NSTEP))
  close(111)

  ! calculate inner product in data space --- <F dm, F dm>
  filename="Ux_file_single.bin"
  open(111,file="./SEM/"//trim(adjustl(filename))//".adj",access='direct',recl=240+4*NSTEP,iostat = ios)
           if (ios /= 0) stop 'file containing adjoint sources does not exist'
  irec=1
  DATASPACE=0.0
  do while(ios==0)
     read(111,rec=irec,iostat=ios) r4head,adj
              if (ios /= 0) cycle
     DATASPACE=DATASPACE+sum(adj(:)*adj(:))*DT
     irec=irec+1
  enddo
  close(111)

  !!! acoustic simulation --- only one component seismograms (Uz not used)
  !filename="Uz_file_single.bin"
  !open(111,file="./SEM/"//trim(adjustl(filename))//".adj",access='direct',recl=240+4*NSTEP,iostat = ios)
  !         if (ios /= 0) stop 'file containing adjoint sources does not exist'
  !irec=1
  !do while(ios==0)
  !   read(111,rec=irec,iostat=ios) r4head,adj
  !            if (ios /= 0) cycle
  !   DATASPACE=DATASPACE+sum(adj(:)*adj(:))*DT
  !   irec=irec+1
  !enddo
  !close(111)

  ! calculate inner product in model space --- <F* F dm, dm>
  filename="./DATA/model_initial"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file model_initial does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) iglob,x,z,rho0(ipoint),vp0(ipoint),vs0(ipoint)
  enddo
  close(111)
  mu0(:)=rho0(:)*vs0(:)*vs0(:)
  kappa0(:)=rho0*vp0(:)*vp0(:)-4.0/3.0*mu0(:)

  filename="./DATA/model_target"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file model_target does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) iglob,x,z,rho(ipoint),vp(ipoint),vs(ipoint)
  enddo
  close(111)
  mu(:)=rho(:)*vs(:)*vs(:)
  kappa(:)=rho*vp(:)*vp(:)-4.0/3.0*mu(:)

  filename="./OUTPUT_FILES/x_z_weightLineX_weightLineZ_weightSurface_NSPEC"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file x_z_weightLineX_weightLineZ_weightSurface_NSPEC does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) x,z,weights(ipoint)
  enddo
  close(111)

  filename="./OUTPUT_FILES/proc000000_rho_kappa_mu_kernel.dat"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file proc000000_rho_kappa_kernel.dat does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) x,z,krho(ipoint),kkappa(ipoint),kmu(ipoint)
  enddo
  close(111)

  filename="./OUTPUT_FILES/proc000000_rhop_alpha_beta_kernel.dat"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file proc000000_rhop_c_kernel.dat does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) x,z,krhop(ipoint),kalpha(ipoint),kbeta(ipoint)
  enddo
  close(111)

  !(F dm = data-syn, we back propagate syn-data ==> add a minus sign to kernels)
  MODELSPACE_temp=0.0
  print*, minval(vs0)
  where (vs0>100.0) &
  MODELSPACE_temp=weights*(-krhop)*(rho-rho0)/rho0+ &
                  weights*(-kalpha)*(vp-vp0)/vp0+ & 
                  weights*(-kbeta)*(vs-vs0)/vs0

  filename="./OUTPUT_FILES/proc000000_rho_kappa_kernel.dat"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file proc000000_rho_kappa_kernel.dat does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) x,z,krho(ipoint),kkappa(ipoint)
  enddo
  close(111)

  filename="./OUTPUT_FILES/proc000000_rhop_c_kernel.dat"
  open(111,file=trim(adjustl(filename)),iostat = ios)
           if (ios /= 0) stop 'file proc000000_rhop_c_kernel.dat does not exist'
  do ipoint=1,NPOINTS
     read(111,*,iostat=ios) x,z,krhop(ipoint),kalpha(ipoint)
  enddo
  close(111)

  !(F dm = data-syn, we back propagate syn-data ==> add a minus sign to kernels)
  where (vs0<100.0) &
  MODELSPACE_temp=weights*(-krhop)*(rho-rho0)/rho0+ &
                  weights*(-kalpha)*(vp-vp0)/vp0 

  MODELSPACE=sum(MODELSPACE_temp)
  print*, 'DATASPACE=',DATASPACE
  print*, 'MODELSPACE=',MODELSPACE
  print*, 'relative error=',(DATASPACE-MODELSPACE)/DATASPACE

end program postprocessing
