#!/bin/bash -eu

model=$1
DIR_DAT=$PWD   # where parameter files are stored
DIR_SEM="../../../"  # where excutables are (not the bin)

cd $DIR_SEM
mkdir -p SEM OUTPUT_FILES
rm -rf SEM/* OUTPUT_FILES/*

rm -rf DATA_BACKUP
mv DATA DATA_BACKUP # backup old DATA
cp -r $DIR_DAT DATA # DATA for the benchmark

### data     
cp "DATA/model_target_$model" DATA/model_velocity.dat_input
cp "DATA/model_target_$model" DATA/model_target
sed -e "s#^SIMULATION_TYPE.*#SIMULATION_TYPE = 1 #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file
sed -e "s#^SAVE_FORWARD.*#SAVE_FORWARD = .true. #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file

cp src/specfem2D/specfem2D.F90 src/specfem2D/specfem2D.F90_backup
cp $DIR_DAT/specfem2D.F90 src/specfem2D/specfem2D.F90
mv setup/constants.h setup/constants.h_backup
mv setup/precision.h setup/precision.h_backup
cp $DIR_DAT/constants.h    setup/
cp $DIR_DAT/precision.h    setup/

make
cp bin/* ./
./xmeshfem2D
./xspecfem2D
cp OUTPUT_FILES/pressure_file_single.bin OUTPUT_FILES/Ux_file_single.bin.dat
cp OUTPUT_FILES/pressure_file_single.bin OUTPUT_FILES/Uz_file_single.bin.dat

### syn
cp "DATA/model_initial_$model" DATA/model_velocity.dat_input
cp "DATA/model_initial_$model" DATA/model_initial
sed -e "s#^SIMULATION_TYPE.*#SIMULATION_TYPE = 1 #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file
sed -e "s#^SAVE_FORWARD.*#SAVE_FORWARD = .true. #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file

./xmeshfem2D
./xspecfem2D
cp OUTPUT_FILES/pressure_file_single.bin OUTPUT_FILES/Ux_file_single.bin.syn
cp OUTPUT_FILES/pressure_file_single.bin OUTPUT_FILES/Uz_file_single.bin.syn

### adjoint sources
ifort -assume byterecl DATA/adj_seismogram.f90 -traceback -o ./xadj
./xadj
rm ./xadj

### adjoint
sed -e "s#^SIMULATION_TYPE.*#SIMULATION_TYPE = 2 #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file
sed -e "s#^SAVE_FORWARD.*#SAVE_FORWARD = .false. #g" < DATA/Par_file > tmp
mv tmp DATA/Par_file

./xmeshfem2D
./xspecfem2D

### postprocessing
ifort -assume byterecl DATA/postprocessing.f90 -traceback -o ./xpostprocessing
./xpostprocessing
rm ./xpostprocessing

### restore original DATA, setup
rm -rf DATA
mv DATA_BACKUP DATA
mv src/specfem2D/specfem2D.F90_backup src/specfem2D/specfem2D.F90
mv setup/constants.h_backup setup/constants.h
mv setup/precision.h_backup setup/precision.h

